"""
Tamga - A modern, async-capable Python logging utility
"""

from .main import Tamga

__version__ = "1.0.1"
__author__ = "Doğukan Ürker"
__email__ = "dogukanurker@icloud.com"
__license__ = "MIT"

__all__ = ["Tamga"]
